# settings. tune it with your data.
VM_NAME="ubuntu-ema"
INTERFACE="enp1s0"
USER_NAME="vmadm"
USER_PASSWORD="vmadm"

DOWNLOADED_IMG_FILE="jammy-server-cloudimg-amd64-disk-kvm.img"
SRC_IMG_FILE="$VM_NAME-src.img"
DST_IMG_FILE="$VM_NAME.qcow2"
DST_IMG_CFG_FILE="$VM_NAME-seed.qcow2"
VM_IMAGES_DIR="/var/lib/libvirt/images"

# network config file. could be tuned too.
NETWORK_CONFIG_FILE="$VM_NAME-network.cfg"
NETWORK_CONFIG=$(cat <<-END
version: 2
ethernets:
  $INTERFACE:
     dhcp4: true
     dhcp6: false
END
)
# user config file. could be tuned too.
USER_CONFIG_FILE="$VM_NAME-user.cfg"
USER_CONFIG=$(cat <<-END
#cloud-config
hostname: $VM_NAME
manage_etc_hosts: true
users:
  - name: $USER_NAME
    sudo: ALL=(ALL) NOPASSWD:ALL
    groups: users, admin
    home: /home/$USER_NAME
    shell: /bin/bash
    lock_passwd: false
ssh_pwauth: true
disable_root: false
chpasswd:
  list: |
    $USER_NAME:$USER_PASSWORD
  expire: false
final_message: "The system is finally up, after \$UPTIME seconds"
END
)

# create random bytes for MAC-address
A=$(od -A n -t d -N 1 /dev/urandom |tr -d ' ')
B=$(od -A n -t d -N 1 /dev/urandom |tr -d ' ')
C=$(od -A n -t d -N 1 /dev/urandom |tr -d ' ')
MAC_ADDR=$(printf '52:54:00:%02x:%02x:%02x' $A  $B $C)

# check before start.
if [ -f $DOWNLOADED_IMG_FILE ]
then
    echo "OK. Downloaded image file exists."
else
    echo "Error! Downloaded image is not found."
    exit;
fi

if [ -d $VM_IMAGES_DIR ]
then
    echo "OK. Folder of VMs images exists."
else
    echo "Error! Folder of VMs images is not found."
    exit;
fi
exit;

# actions
cp $DOWNLOADED_IMG_FILE "$VM_IMAGES_DIR/$SRC_IMG_FILE"
qemu-img create -F qcow2 -b "$VM_IMAGES_DIR/$SRC_IMG_FILE" -f qcow2 "$VM_IMAGES_DIR/$DST_IMG_FILE" 20G
echo "$USER_CONFIG" >> "$USER_CONFIG_FILE"
echo "$NETWORK_CONFIG" >> "$NETWORK_CONFIG_FILE"
cloud-localds -v --network-config="$NETWORK_CONFIG_FILE" "$VM_IMAGES_DIR/$DST_IMG_CFG_FILE" "$USER_CONFIG_FILE"
virt-install \
  --connect qemu:///system \
  --virt-type kvm \
  --name $VM_NAME \
  --ram 4096 \
  --vcpus 2 \
  --os-variant ubuntu20.04 \
  --disk path="$VM_IMAGES_DIR/$DST_IMG_FILE",device=disk \
  --disk path="$VM_IMAGES_DIR/$DST_IMG_CFG_FILE",device=disk \
  --network network=default,model=virtio,mac="$MAC_ADDR" \
  --graphics spice --video qxl --channel spicevmc \
  --import \
  --noautoconsole
  
